#!/usr/bin/env python
#
# Copyright 2008 Mike Wakerly <opensource@hoho.com>
#
# This file is part of the Pykeg package of the Kegbot project.
# For more information on Pykeg or Kegbot, see http://kegbot.org/
#
# Pykeg is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#
# Pykeg is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Pykeg.  If not, see <http://www.gnu.org/licenses/>.

"""Kegweb main views."""

import datetime

from django.conf import settings
from django.contrib.auth.decorators import login_required
from django.http import Http404
from django.shortcuts import get_object_or_404
from django.shortcuts import render_to_response
from django.template import RequestContext
from django.views.decorators.cache import cache_page
from django.views.generic.list_detail import object_detail
from django.views.generic.list_detail import object_list
from django.views.generic.simple import redirect_to
from django.contrib.sites.models import Site
from django.http import HttpResponseRedirect

from kegbot.util import kbjson

from pykeg.core import models
from pykeg.proto import protolib

from pykeg.web.kegweb import forms
from pykeg.web.kegweb import models as kegweb_models
from pykeg.web.kegweb import signals

### main views

@cache_page(30)
def index(request):
  context = RequestContext(request)
  try:
    page = kegweb_models.Page.objects.get(name__exact='MainPage',
                                          status__exact='published')
  except kegweb_models.Page.DoesNotExist:
    page = None
  context['page_node'] = page
  context['taps'] = request.kbsite.taps.all()

  try:
    session = request.kbsite.sessions.latest()
    if session.IsActiveNow():
      context['current_session'] = session
  except models.DrinkingSession.DoesNotExist:
    pass

  recent_images = models.Picture.objects.filter(
      site=request.kbsite,
      drink__isnull=False).order_by('-time')[:9]
  context['recent_images'] = recent_images

  events = request.kbsite.events.all()[:10]
  context['initial_events'] = kbjson.dumps([protolib.ToDict(e, full=True) for e in events],
      indent=None)

  sessions = request.kbsite.sessions.all().order_by('-seqn')[:10]
  context['initial_sessions'] = kbjson.dumps([protolib.ToDict(s, full=True) for s in sessions],
      indent=None)

  return render_to_response('index.html', context)

@cache_page(30)
def system_stats(request):
  stats = request.kbsite.GetStats()
  context = RequestContext(request, {
    'stats': stats,
  })

  top_drinkers = []
  for drinkervol in stats.get('volume_by_drinker', []):
    username = drinkervol.username
    vol = drinkervol.volume_ml
    try:
      user = models.User.objects.get(username=username)
    except models.User.DoesNotExist:
      continue  # should not happen
    top_drinkers.append((vol, user))
  top_drinkers.sort(reverse=True)

  context['top_drinkers'] = top_drinkers[:10]

  return render_to_response('kegweb/system-stats.html', context)


### object lists and detail (generic views)

def user_list(request):
  user_list = models.User.objects.all()
  return object_list(request,
      queryset=user_list,
      template_object_name='drinker',
      template_name='kegweb/drinker_list.html')

def user_detail(request, username):
  user = get_object_or_404(models.User, username=username)
  try:
    stats = models.UserStats.objects.get(site=request.kbsite, user=user).stats
  except models.UserStats.DoesNotExist:
    stats = None

  # TODO(mikey): Limit to "recent" sessions for now.
  # Bug: https://github.com/Kegbot/kegbot/issues/37
  chunks = models.UserSessionChunk.objects.filter(site=request.kbsite,
      user=user).select_related(depth=1)[:10]
  drinks = user.drinks.filter(site=request.kbsite)

  context = RequestContext(request, {
      'drinks': drinks,
      'chunks': chunks,
      'stats': stats,
      'drinker': user})
  return render_to_response('kegweb/drinker_detail.html', context)

def user_detail_by_id(request, user_id):
  try:
    user = models.User.objects.get(pk=user_id)
  except models.User.DoesNotExist:
    raise Http404
  return redirect_to(request, url='/drinker/'+user.username)

def keg_list(request):
  all_kegs = request.kbsite.kegs.all().order_by('-id')
  return object_list(request,
      queryset=all_kegs,
      template_object_name='keg',
      template_name='kegweb/keg_list.html')

@cache_page(30)
def keg_detail(request, keg_id):
  keg = get_object_or_404(models.Keg, site=request.kbsite, seqn=keg_id)
  sessions = keg.Sessions()
  context = RequestContext(request, {
    'keg': keg,
    'stats': keg.GetStats(),
    'sessions': sessions})
  return render_to_response('kegweb/keg_detail.html', context)

def short_drink_detail(request, drink_id):
  url = request.kbsite.full_url() + '/drinks/' + str(drink_id)
  return HttpResponseRedirect(url)

def short_session_detail(request, session_id):
  session = get_object_or_404(models.DrinkingSession, site=request.kbsite,
      seqn=session_id)
  url = session.get_absolute_url()
  return HttpResponseRedirect(url)

def drink_detail(request, drink_id):
  drink = get_object_or_404(models.Drink, site=request.kbsite, seqn=drink_id)
  context = RequestContext(request, {'drink': drink})
  return render_to_response('kegweb/drink_detail.html', context)

### auth

def webauth(request):
  context = {}
  return render_to_response('kegweb/webauth.html', context)

@login_required
def claim_token(request):
  if request.method == 'POST':
    form = forms.ClaimTokenForm(request.POST)

    if form.is_valid():
      user = form.cleaned_data['user']
      token = form.cleaned_data['token']
      # TODO(mikey): non-superusers should only be able to claim tokens for
      # their own account.
      token.user = user
      token.save()
  else:
    form = forms.ClaimTokenForm()

  context = RequestContext(request)
  context['form'] = form
  return render_to_response('kegweb/claim_token.html', context)

def session_detail(request, year, month, day, seqn, slug):
  session = get_object_or_404(models.DrinkingSession, site=request.kbsite, seqn=seqn)
  context = RequestContext(request, {
    'session': session,
    'stats': session.GetStats(),
  })
  return render_to_response('kegweb/session_detail.html', context)





# Copyright 2012 Mike Wakerly <opensource@hoho.com>
#
# This file is part of the Pykeg package of the Kegbot project.
# For more information on Pykeg or Kegbot, see http://kegbot.org/
#
# Pykeg is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#
# Pykeg is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Pykeg.  If not, see <http://www.gnu.org/licenses/>.

from imagekit.models import ImageSpec
from imagekit.processors import Adjust
from imagekit.processors import resize

resized = ImageSpec(
    processors=[resize.ResizeToFit(1024, 1024)],
    image_field='image',
    format='JPEG',
    options={'quality': 90},
)

small_resized = ImageSpec(
    processors=[resize.ResizeToFit(256, 256)],
    image_field='image',
    format='JPEG',
    options={'quality': 100},
)

thumbnail = ImageSpec(
    processors=[Adjust(contrast=1.2, sharpness=1.1), resize.SmartResize(128, 128)],
    image_field='image',
    format='JPEG',
    options={'quality': 100},
)

small_thumbnail = ImageSpec(
    processors=[Adjust(contrast=1.2, sharpness=1.1), resize.SmartResize(32, 32)],
    image_field='image',
    format='JPEG',
    options={'quality': 100},
)



from django.conf import settings
from django.contrib.auth.decorators import login_required
from django.http import Http404
from django.shortcuts import render_to_response
from django.template import RequestContext
from django.views.generic.simple import redirect_to

from pykeg.contrib.facebook import fbutil
from pykeg.contrib.facebook import models
from pykeg.web.contrib.facebook import forms

import facebook

@login_required
def update_perms(request):
  context = RequestContext(request)
  models.FacebookSession.get_session(request)
  # TODO(mikey): record permissions in table.
  raise Http404

@login_required
def account_settings(request):
  context = RequestContext(request)
  profile = fbutil.profile_for_user(request.user)
  if not profile:
    return render_to_response('contrib/facebook/link.html', context)

  settings = profile.settings.all()[0]
  form = forms.FacebookSettingsForm(instance=settings)
  if request.method == 'POST':
    form = forms.FacebookSettingsForm(request.POST, instance=settings)
    if form.is_valid():
      profile = fbutil.profile_for_user(request.user)
      if not profile:
        raise ValueError, "Bad profile"
      settings = form.save(commit=False)
      settings.profile = profile
      settings.save()
  context['settings_form'] = form
  return render_to_response('contrib/facebook/settings.html', context)

@login_required
def status_update(request):
  if request.method == 'POST':
    form = forms.WallPostForm(request.POST)
    if form.is_valid():
      fbutil.stream_publish(request.user, message=form.cleaned_data['message'])
  context = {'form': forms.WallPostForm()}
  return render_to_response('contrib/facebook/status-update.html', context)



# Copyright 2010 Mike Wakerly <opensource@hoho.com>
#
# This file is part of the Pykeg package of the Kegbot project.
# For more information on Pykeg or Kegbot, see http://kegbot.org/
#
# Pykeg is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#
# Pykeg is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Pykeg.  If not, see <http://www.gnu.org/licenses/>.

"""Kegweb API client."""

import datetime
import sys
import urllib2
from urllib import urlencode
from urllib2 import HTTPError
from urllib2 import URLError


from kegbot.util import kbjson

import gflags

gflags.DEFINE_float('krest_timeout', 10.0,
    'Socket timeout, in seconds, for Kegbot web API operations. '
    'Note that this timeout only applies to blocking socket operations '
    '(such as opening a connection) and not I/O.')

FLAGS = gflags.FLAGS

_DEFAULT_URL = 'http://localhost:8000/api/'
_DEFAULT_KEY = ''
try:
  from pykeg import settings
  if hasattr(settings, 'KEGWEB_BASE_URL'):
    _DEFAULT_URL = '%s/api/' % getattr(settings, 'KEGWEB_BASE_URL')
  if hasattr(settings, 'KEGWEB_API_KEY'):
    _DEFAULT_KEY = settings.KEGWEB_API_KEY
except ImportError:
  # Non-fatal if we can't load settings.
  pass

gflags.DEFINE_string('api_url', _DEFAULT_URL,
    'Base URL for the Kegweb HTTP api.')

gflags.DEFINE_string('api_key', _DEFAULT_KEY,
    'Access key for the Kegweb HTTP api.')

### begin common

class Error(Exception):
  """An error occurred."""
  HTTP_CODE = 400
  def Message(self):
    if self.message:
      return self.message
    m = self.__class__.__doc__
    m = m.split('\n', 1)[0]
    return m

class NotFoundError(Error):
  """The requested object could not be found."""
  HTTP_CODE = 404

class ServerError(Error):
  """The server had a problem fulfilling your request."""
  HTTP_CODE = 500

class BadRequestError(Error):
  """The request was incompleted or malformed."""
  HTTP_CODE = 401

class NoAuthTokenError(Error):
  """An api_key is required."""
  HTTP_CODE = 401

class BadApiKeyError(Error):
  """The api_key given is invalid."""
  HTTP_CODE = 401

class PermissionDeniedError(Error):
  """The api_key given does not have permission for this resource."""
  HTTP_CODE = 401

MAP_NAME_TO_EXCEPTION = dict((c.__name__, c) for c in Error.__subclasses__())

def ErrorCodeToException(code, message=None):
  cls = MAP_NAME_TO_EXCEPTION.get(code, Error)
  return cls(message)

def decode_response(response_data):
  """Decodes the string `response_data` as a JSON response.

  For normal responses, the return value is the Python JSON-decoded 'result'
  field of the response.  If the response is an error, a RemoteError exception
  is raised.
  """
  # Decode JSON.
  try:
    d = kbjson.loads(response_data)
  except ValueError, e:
    raise ServerError('Malformed response: %s' % e)

  if 'error' in d:
    # Response had an error: translate to exception.
    err = d['error']
    code = err.get('code', 500)
    message = err.get('message', None)
    e = ErrorCodeToException(code, message)
    raise e
  elif 'object' in d or 'objects' in d:
    # Response was OK, return the result.
    return d
  else:
    # WTF?
    raise ValueError('Invalid response from server: missing result or error')

### end common

class KrestClient:
  """Kegweb RESTful API client."""
  def __init__(self, api_url=None, api_key=None):
    if api_url is None:
      api_url = FLAGS.api_url
    if api_key is None:
      api_key = FLAGS.api_key
    self._api_url = api_url
    self._api_key = api_key
    self._opener = urllib2.build_opener(KrestHTTPErrorProcessor)

  def _Encode(self, s):
    return unicode(s).encode('utf-8')

  def _EncodePostData(self, post_data):
    if not post_data:
      return None
    return urlencode(dict(((k, self._Encode(v)) for k, v in
        post_data.iteritems() if v is not None)))

  def _GetURL(self, endpoint, params=None):
    param_str = ''
    if params:
      param_str = '?%s' % urlencode(params)

    base = self._api_url.rstrip('/')

    # Strip both leading and trailing slash from endpoint: single leading and
    # trailing slashes will be added by the string formatter.  (The trailing
    # slash is required for POSTs to Django.)
    endpoint = endpoint.strip('/')
    return '%s/%s/%s' % (base, endpoint, param_str)

  def SetAuthToken(self, api_key):
    self._api_key = api_key

  def DoGET(self, endpoint, params=None):
    """Issues a GET request to the endpoint, and retuns the result.

    Keyword arguments are passed to the endpoint as GET arguments.

    For normal responses, the return value is the Python JSON-decoded 'object'
    or 'objects' field of the response.  If the response is an error, a
    RemoteError exception is raised.

    If there was an error contacting the server, or in parsing its response, a
    ServerError is raised.
    """
    return self._FetchResponse(endpoint, params=params)

  def DoPOST(self, endpoint, post_data, params=None):
    """Issues a POST request to the endpoint, and returns the result.

    For normal responses, the return value is the Python JSON-decoded 'object'
    or 'objects' field of the response.  If the response is an error, a
    RemoteError exception is raised.

    If there was an error contacting the server, or in parsing its response, a
    ServerError is raised.
    """
    return self._FetchResponse(endpoint, params=params, post_data=post_data)

  def _FetchResponse(self, endpoint, params=None, post_data=None):
    """Issues a POST or GET request, depending on the arguments."""
    if params is None:
      params = {}
    else:
      params = params.copy()

    # If we have an api token, attach it.  Prefer to attach it to POST data, but
    # use GET if there is no POST data.
    if self._api_key:
      if post_data:
        post_data['api_key'] = self._api_key
      else:
        params['api_key'] = self._api_key

    url = self._GetURL(endpoint, params=params)
    encoded_post_data = self._EncodePostData(post_data)

    try:
      # Issue a GET or POST (urlopen will decide based on encoded_post_data).
      response_data = self._opener.open(url, data=encoded_post_data,
          timeout=FLAGS.krest_timeout).read()
    except HTTPError, e:
      raise ServerError('Caused by: %s' % e)
    except URLError, e:
      raise ServerError('URL Error, reason: %s' % e.reason)

    return decode_response(response_data)

  def RecordDrink(self, tap_name, ticks, volume_ml=None, username=None,
      pour_time=None, duration=0, auth_token=None, spilled=False, shout=''):
    endpoint = '/taps/%s' % tap_name
    post_data = {
      'tap_name': tap_name,
      'ticks': ticks,
      'volume_ml': volume_ml,
      'username': username,
      'auth_token': auth_token,
      'duration': duration,
      'spilled': spilled,
    }
    if pour_time:
      post_data['pour_time'] = int(pour_time.strftime('%s'))
      post_data['now'] = int(datetime.datetime.now().strftime('%s'))
    if shout:
      post_data['shout'] = shout
    return self.DoPOST(endpoint, post_data=post_data).object

  def CancelDrink(self, seqn, spilled=False):
    endpoint = '/cancel-drink'
    post_data = {
      'id': seqn,
      'spilled': spilled,
    }
    return self.DoPOST(endpoint, post_data=post_data).object

  def LogSensorReading(self, sensor_name, temperature, when=None):
    endpoint = '/thermo-sensors/%s' % (sensor_name,)
    post_data = {
      'temp_c': float(temperature),
    }
    # TODO(mikey): include post data
    return self.DoPOST(endpoint, post_data=post_data).object

  def TapStatus(self):
    """Gets the status of all taps."""
    return self.DoGET('taps').objects

  def GetToken(self, auth_device, token_value):
    url = 'auth-tokens/%s/%s' % (auth_device, token_value)
    try:
      return self.DoGET(url).object
    except ServerError, e:
      raise NotFoundError(e)

  def LastDrinks(self):
    """Gets a list of the most recent drinks."""
    return self.DoGET('last-drinks').objects

  def AllDrinks(self):
    """Gets a list of all drinks."""
    return self.DoGET('drinks').objects

  def AllSoundEvents(self):
    """Gets a list of all drinks."""
    return self.DoGET('sound-events').objects

class KrestHTTPErrorProcessor(urllib2.BaseHandler):
  def _handle_error(self, request, response, code, msg, hdrs):
    data = response.read()
    decode_response(data)

  http_error_401 = _handle_error
  http_error_403 = _handle_error
  http_error_404 = _handle_error
  http_error_405 = _handle_error
  http_error_500 = _handle_error

def main():
  c = KrestClient()

  print '== record a drink =='
  print c.RecordDrink('kegboard.flow0', 2200)
  print ''

  print '== tap status =='
  for t in c.TapStatus().taps:
    print t
    print ''

  print '== last drinks =='
  for d in c.LastDrinks().drinks:
    print d
    print ''

if __name__ == '__main__':
  FLAGS(sys.argv)
  main()



from django.conf.urls.defaults import *

urlpatterns = patterns('pykeg.web.kegadmin.views',
    ### main page
    url(r'^$', 'kegadmin_main', name='kegadmin-main'),
    url(r'^taps/$', 'tap_list', name='kegadmin-taps'),
    url(r'^taps/(?P<tap_id>\d+)/$', 'edit_tap', name='kegadmin-edit-tap'),
    url(r'^backup-restore/$', 'backup_restore', name='kegadmin-backup-restore'),
    url(r'^backup-restore/dump/$', 'generate_backup', name='kegadmin-get-backup'),
    url(r'^do-end-keg/$', 'do_end_keg', name='kegadmin-do-end-keg'),
    url(r'^connections/', include('pykeg.connections.urls')),
    url(r'^edit-connections/$', 'connections', name='kegadmin-connections'),
    url(r'^logs/$', 'logs', name='kegadmin-logs'),
)

urlpatterns += patterns('pykeg.connections.twitter.views',
    url('^redirect/$', 'site_twitter_redirect', name='site_twitter_redirect'),
    url('^callback/$', 'site_twitter_callback', name='site_twitter_callback'),
)



# Copyright 2012 Mike Wakerly <opensource@hoho.com>
#
# This file is part of the Pykeg package of the Kegbot project.
# For more information on Pykeg or Kegbot, see http://kegbot.org/
#
# Pykeg is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#
# Pykeg is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Pykeg.  If not, see <http://www.gnu.org/licenses/>.

"""Celery tasks for Twitter."""

from . import util
from pykeg.connections import common

import tweepy
from django.contrib.sites.models import Site

from celery.task import task

logger = common.get_logger(__name__)

@task
def tweet_event(event):
  if common.is_stale(event.time):
    logger.info('Event is stale, ignoring: %s' % str(event))
    return
  if event.kind not in ('session_joined', 'drink_poured'):
    logger.info('Event is not tweetable: %s' % event.kind)
    return
  kbvars = _get_vars(event)
  do_user_tweet(event, kbvars)
  do_system_tweet(event, kbvars)

def add_hashtag(tweet):
  if len(tweet) <= (140 - len(util.HASHTAG) - 1):
    tweet = '%s %s' % (tweet, util.HASHTAG)
  return tweet

def _get_vars(event):
  base_url = 'http://%s/%s' % (Site.objects.get_current().domain, event.site.url())
  name = ''
  if event.user:
    name = event.user.username
  session_url = ''
  drink_url = ''

  if event.drink:
    session_url = '%s/%s' % (base_url, event.drink.session.get_absolute_url())
    drink_url = event.drink.ShortUrl()

  beer_name = ''
  if event.drink.keg and event.drink.keg.type:
    beer_name = event.drink.keg.type.name

  drink_size = ''
  if event.drink:
    drink_size = '%.1foz' % event.drink.Volume().InOunces()

  kbvars = {
    'kb_name': event.site.settings.title,
    'name': name,
    'kb_url': base_url,
    'drink_url': drink_url,
    'session_url': session_url,
    'drink_size': drink_size,
    'beer_name': beer_name,
  }
  return kbvars


def do_system_tweet(event, kbvars):
  """Sends a tweet using the system's account (if any)."""
  system_profile = util.get_site_profile(event.site)
  if not system_profile:
    logger.info('No system twitter profile, skipping system tweet')
    return

  user = event.user
  if user:
    user_profile = util.get_user_profile(user)
    if user_profile:
      name = '@%s' % (user_profile.settings.twitter_name,)
    elif not system_profile.settings.post_unlinked:
      logger.info('Tweets for unlinked users are disabled')
      return
    else:
      name = user.username
  elif not system_profile.settings.post_unauthenticated:
    logger.info('Tweets for anonymous pours are disabled.')
    return
  else:
    name = 'Someone'

  kbvars['name'] = name

  system_api = util.get_api(system_profile.oauth_token,
      system_profile.oauth_token_secret)

  tweet = None
  if event.kind == 'session_joined':
    tweet = util.DEFAULT_SYSTEM_SESSION_JOINED_TEMPLATE % kbvars
  elif event.kind == 'drink_poured':
    tweet = util.DEFAULT_SYSTEM_DRINK_POURED_TEMPLATE % kbvars

  if tweet:
    tweet = add_hashtag(tweet)
    logger.info('Sending system tweet: %s' % tweet)
    try:
      system_api.update_status(tweet)
    except tweepy.TweepError, e:
      logger.error('Error posting tweet: %s' % str(e))

def do_user_tweet(event, kbvars):
  """Sends a tweet using the user's account (if any)."""
  user = event.user
  if not user:
    logger.info('No user for this event, no user tweet possible: %s' % str(event))
    return

  profile = util.get_user_profile(user)
  if not profile or not profile.settings.enabled:
    logger.info('User has disabled Twitter: %s' % user)
    return

  kind = event.kind
  tweet = None
  if kind == 'drink_poured':
    if not profile.settings.post_drink_poured:
      logger.info('User has disabled drink pour tweets: %s' % event.user)
      return
    tweet = util.DEFAULT_USER_DRINK_POURED_TEMPLATE % kbvars
  elif kind == 'session_joined':
    if not profile.settings.post_session_joined:
      logger.info('User has disabled session-join tweets: %s' % event.user)
      return
    tweet = util.DEFAULT_USER_SESSION_JOINED_TEMPLATE % kbvars

  user_api = util.get_api(profile.access_token.oauth_token,
      profile.access_token.oauth_token_secret)

  if tweet:
    tweet = add_hashtag(tweet)
    logger.info('Sending user tweet: %s' % tweet)
    try:
      user_api.update_status(tweet)
    except tweepy.TweepError, e:
      logger.error('Error posting tweet: %s' % str(e))



from django import forms
from django.utils.encoding import force_unicode

### Numeric ChoiceField with "Other" option.
# Based on: http://sciyoshi.com/2008/7/django-choicefield-other-choice/

class ChoiceWithOtherRenderer(forms.RadioSelect.renderer):
    """RadioFieldRenderer that renders its last choice with a placeholder."""
    def __init__(self, *args, **kwargs):
        super(ChoiceWithOtherRenderer, self).__init__(*args, **kwargs)
        self.choices, self.other = self.choices[:-1], self.choices[-1]

    def __iter__(self):
        for input in super(ChoiceWithOtherRenderer, self).__iter__():
            yield input
        id = '%s_%s' % (self.attrs['id'], self.other[0]) if 'id' in self.attrs else ''
        label_for = ' for="%s"' % id if id else ''
        checked = '' if not force_unicode(self.other[0]) == self.value else 'checked="true" '
        yield '<label%s><input type="radio" id="%s" value="%s" name="%s" %s/> %s</label> %%s' % (
            label_for, id, self.other[0], self.name, checked, self.other[1])

class ChoiceWithOtherWidget(forms.MultiWidget):
  """MultiWidget for use with ChoiceWithOtherField."""
  def __init__(self, choices):
    widgets = [
        forms.RadioSelect(choices=choices, renderer=ChoiceWithOtherRenderer),
        forms.TextInput
    ]
    super(ChoiceWithOtherWidget, self).__init__(widgets)

  def decompress(self, value):
    if not value:
        return [None, None]
    return value

  def format_output(self, rendered_widgets):
    """Format the output by substituting the "other" choice into the first widget."""
    return rendered_widgets[0] % rendered_widgets[1]

class ChoiceWithOtherField(forms.MultiValueField):
  def __init__(self, *args, **kwargs):
    fields = [
        forms.ChoiceField(widget=forms.RadioSelect(renderer=ChoiceWithOtherRenderer), *args, **kwargs),
        forms.FloatField(required=False, min_value=0.000001, max_value=1.0)
    ]
    widget = ChoiceWithOtherWidget(choices=kwargs['choices'])
    kwargs.pop('choices')
    self._was_required = kwargs.pop('required', True)
    kwargs['required'] = False
    super(ChoiceWithOtherField, self).__init__(widget=widget, fields=fields, *args, **kwargs)

  def compress(self, value):
    if self._was_required and not value or value[0] in (None, ''):
        raise forms.ValidationError(self.error_messages['required'])
    if not value:
        return [None, u'']
    return (value[0], value[1] if force_unicode(value[0]) == force_unicode(self.fields[0].choices[-1][0]) else u'')




# Copyright 2010 Mike Wakerly <opensource@hoho.com>
#
# This file is part of the Pykeg package of the Kegbot project.
# For more information on Pykeg or Kegbot, see http://kegbot.org/
#
# Pykeg is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#
# Pykeg is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Pykeg.  If not, see <http://www.gnu.org/licenses/>.

import sys

def progbar(title, pos, total, width=40):
  """Prints a progress bar to stdout.

  Args
    title: title to show next to progress bar
    pos: current position (integer)
    total: total positions (integer)
    width: width of the progres bar, in characters
  """
  if not total:
    chars = width
  else:
    chars = int((float(pos)/total)*width)
  rem = width - chars
  inner = '+'*chars + ' '*rem
  sys.stdout.write('%-30s  [%s] %i/%i\r' % (title, inner, pos, total))
  sys.stdout.flush()





# Copyright 2013 Mike Wakerly <opensource@hoho.com>
#
# This file is part of the Pykeg package of the Kegbot project.
# For more information on Pykeg or Kegbot, see http://kegbot.org/
#
# Pykeg is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#
# Pykeg is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Pykeg.  If not, see <http://www.gnu.org/licenses/>.

from functools import wraps
from django.conf import settings
from django.contrib import messages
from django.contrib.auth import authenticate
from django.contrib.auth import login
from django.core.urlresolvers import reverse
from django.http import Http404
from django.shortcuts import render_to_response
from django.template import RequestContext
from django.views.decorators.cache import never_cache
from django.views.generic.simple import redirect_to

from .forms import AdminUserForm
from .forms import MiniSiteSettingsForm

def setup_view(f):
  """Decorator for setup views."""
  def new_function(*args, **kwargs):
    request = args[0]
    if request.kbsite and request.kbsite.is_setup:
      if not request.user.is_active or not request.user.is_admin:
        raise Http404('Admin user is required to modify existing site.')
    return f(*args, **kwargs)
  return wraps(f)(new_function)

@setup_view
@never_cache
def start(request):
  context = RequestContext(request)
  return render_to_response('setup_wizard/start.html', context)

@setup_view
@never_cache
def site_settings(request):
  context = RequestContext(request)
  form = MiniSiteSettingsForm(instance=request.kbsite.settings)
  if request.method == 'POST':
    form = MiniSiteSettingsForm(request.POST, instance=request.kbsite.settings)
    if form.is_valid():
      form.save()
      messages.success(request, 'Settings saved!')
      result_url = reverse('setup_admin', args=[request.kbsite.url()])
      return redirect_to(request, result_url)
  context['form'] = form
  return render_to_response('setup_wizard/site_settings.html', context)

@setup_view
@never_cache
def admin(request):
  context = RequestContext(request)
  form = AdminUserForm()
  if request.method == 'POST':
    form = AdminUserForm(request.POST)
    if form.is_valid():
      form.save()
      user = authenticate(username=form.cleaned_data.get('username'),
          password=form.cleaned_data.get('password'))
      if user:
        login(request, user)
      result_url = reverse('setup_finish', args=[request.kbsite.url()])
      return redirect_to(request, result_url)
  context['form'] = form
  return render_to_response('setup_wizard/admin.html', context)

@setup_view
@never_cache
def finish(request):
  context = RequestContext(request)
  if request.method == 'POST':
    request.kbsite.is_setup = True
    request.kbsite.save()
    messages.success(request, 'Tip: Install a new Keg in Admin: Taps')
    result_url = reverse('kb-home', args=[request.kbsite.url()])
    return redirect_to(request, result_url)
  return render_to_response('setup_wizard/finish.html', context)



# -*- coding: utf-8 -*-
import datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models


class Migration(SchemaMigration):

    def forwards(self, orm):
        # Adding model 'SiteTwitterProfile'
        db.create_table('twitter_sitetwitterprofile', (
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('site', self.gf('django.db.models.fields.related.OneToOneField')(related_name='twitter_profile', unique=True, to=orm['core.KegbotSite'])),
            ('twitter_name', self.gf('django.db.models.fields.CharField')(max_length=32)),
            ('twitter_id', self.gf('django.db.models.fields.PositiveIntegerField')()),
            ('oauth_token', self.gf('django.db.models.fields.CharField')(max_length=80)),
            ('oauth_token_secret', self.gf('django.db.models.fields.CharField')(max_length=80)),
            ('enabled', self.gf('django.db.models.fields.BooleanField')(default=False)),
        ))
        db.send_create_signal('twitter', ['SiteTwitterProfile'])

        # Adding model 'SiteTwitterSettings'
        db.create_table('twitter_sitetwittersettings', (
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('enabled', self.gf('django.db.models.fields.BooleanField')(default=True)),
            ('post_session_joined', self.gf('django.db.models.fields.BooleanField')(default=True)),
            ('post_drink_poured', self.gf('django.db.models.fields.BooleanField')(default=False)),
            ('profile', self.gf('django.db.models.fields.related.OneToOneField')(related_name='settings', unique=True, to=orm['twitter.SiteTwitterProfile'])),
        ))
        db.send_create_signal('twitter', ['SiteTwitterSettings'])

        # Adding model 'TwitterProfile'
        db.create_table('twitter_twitterprofile', (
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('user', self.gf('django.db.models.fields.related.ForeignKey')(to=orm['auth.User'], unique=True)),
            ('site', self.gf('django.db.models.fields.related.ForeignKey')(to=orm['sites.Site'])),
            ('twitter_id', self.gf('django.db.models.fields.PositiveIntegerField')()),
        ))
        db.send_create_signal('twitter', ['TwitterProfile'])

        # Adding model 'TwitterRequestToken'
        db.create_table('twitter_twitterrequesttoken', (
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('profile', self.gf('django.db.models.fields.related.OneToOneField')(related_name='request_token', unique=True, to=orm['twitter.TwitterProfile'])),
            ('oauth_token', self.gf('django.db.models.fields.CharField')(max_length=80)),
            ('oauth_token_secret', self.gf('django.db.models.fields.CharField')(max_length=80)),
        ))
        db.send_create_signal('twitter', ['TwitterRequestToken'])

        # Adding model 'TwitterAccessToken'
        db.create_table('twitter_twitteraccesstoken', (
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('profile', self.gf('django.db.models.fields.related.OneToOneField')(related_name='access_token', unique=True, to=orm['twitter.TwitterProfile'])),
            ('oauth_token', self.gf('django.db.models.fields.CharField')(max_length=80)),
            ('oauth_token_secret', self.gf('django.db.models.fields.CharField')(max_length=80)),
        ))
        db.send_create_signal('twitter', ['TwitterAccessToken'])

        # Adding model 'TwitterSettings'
        db.create_table('twitter_twittersettings', (
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('enabled', self.gf('django.db.models.fields.BooleanField')(default=True)),
            ('post_session_joined', self.gf('django.db.models.fields.BooleanField')(default=True)),
            ('post_drink_poured', self.gf('django.db.models.fields.BooleanField')(default=False)),
            ('profile', self.gf('django.db.models.fields.related.OneToOneField')(related_name='settings', unique=True, to=orm['twitter.TwitterProfile'])),
            ('twitter_name', self.gf('django.db.models.fields.CharField')(max_length=64)),
        ))
        db.send_create_signal('twitter', ['TwitterSettings'])

    def backwards(self, orm):
        # Deleting model 'SiteTwitterProfile'
        db.delete_table('twitter_sitetwitterprofile')

        # Deleting model 'SiteTwitterSettings'
        db.delete_table('twitter_sitetwittersettings')

        # Deleting model 'TwitterProfile'
        db.delete_table('twitter_twitterprofile')

        # Deleting model 'TwitterRequestToken'
        db.delete_table('twitter_twitterrequesttoken')

        # Deleting model 'TwitterAccessToken'
        db.delete_table('twitter_twitteraccesstoken')

        # Deleting model 'TwitterSettings'
        db.delete_table('twitter_twittersettings')

    models = {
        'auth.group': {
            'Meta': {'object_name': 'Group'},
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '80'}),
            'permissions': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['auth.Permission']", 'symmetrical': 'False', 'blank': 'True'})
        },
        'auth.permission': {
            'Meta': {'ordering': "('content_type__app_label', 'content_type__model', 'codename')", 'unique_together': "(('content_type', 'codename'),)", 'object_name': 'Permission'},
            'codename': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'content_type': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['contenttypes.ContentType']"}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '50'})
        },
        'auth.user': {
            'Meta': {'object_name': 'User'},
            'date_joined': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            'email': ('django.db.models.fields.EmailField', [], {'max_length': '75', 'blank': 'True'}),
            'first_name': ('django.db.models.fields.CharField', [], {'max_length': '30', 'blank': 'True'}),
            'groups': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['auth.Group']", 'symmetrical': 'False', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'is_active': ('django.db.models.fields.BooleanField', [], {'default': 'True'}),
            'is_staff': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'is_superuser': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'last_login': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            'last_name': ('django.db.models.fields.CharField', [], {'max_length': '30', 'blank': 'True'}),
            'password': ('django.db.models.fields.CharField', [], {'max_length': '128'}),
            'user_permissions': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['auth.Permission']", 'symmetrical': 'False', 'blank': 'True'}),
            'username': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '30'})
        },
        'contenttypes.contenttype': {
            'Meta': {'ordering': "('name',)", 'unique_together': "(('app_label', 'model'),)", 'object_name': 'ContentType', 'db_table': "'django_content_type'"},
            'app_label': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'model': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '100'})
        },
        'core.kegbotsite': {
            'Meta': {'object_name': 'KegbotSite'},
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'is_active': ('django.db.models.fields.BooleanField', [], {'default': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '64'})
        },
        'sites.site': {
            'Meta': {'ordering': "('domain',)", 'object_name': 'Site', 'db_table': "'django_site'"},
            'domain': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '50'})
        },
        'twitter.sitetwitterprofile': {
            'Meta': {'object_name': 'SiteTwitterProfile'},
            'enabled': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'oauth_token': ('django.db.models.fields.CharField', [], {'max_length': '80'}),
            'oauth_token_secret': ('django.db.models.fields.CharField', [], {'max_length': '80'}),
            'site': ('django.db.models.fields.related.OneToOneField', [], {'related_name': "'twitter_profile'", 'unique': 'True', 'to': "orm['core.KegbotSite']"}),
            'twitter_id': ('django.db.models.fields.PositiveIntegerField', [], {}),
            'twitter_name': ('django.db.models.fields.CharField', [], {'max_length': '32'})
        },
        'twitter.sitetwittersettings': {
            'Meta': {'object_name': 'SiteTwitterSettings'},
            'enabled': ('django.db.models.fields.BooleanField', [], {'default': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'post_drink_poured': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'post_session_joined': ('django.db.models.fields.BooleanField', [], {'default': 'True'}),
            'profile': ('django.db.models.fields.related.OneToOneField', [], {'related_name': "'settings'", 'unique': 'True', 'to': "orm['twitter.SiteTwitterProfile']"})
        },
        'twitter.twitteraccesstoken': {
            'Meta': {'object_name': 'TwitterAccessToken'},
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'oauth_token': ('django.db.models.fields.CharField', [], {'max_length': '80'}),
            'oauth_token_secret': ('django.db.models.fields.CharField', [], {'max_length': '80'}),
            'profile': ('django.db.models.fields.related.OneToOneField', [], {'related_name': "'access_token'", 'unique': 'True', 'to': "orm['twitter.TwitterProfile']"})
        },
        'twitter.twitterprofile': {
            'Meta': {'object_name': 'TwitterProfile'},
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'site': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['sites.Site']"}),
            'twitter_id': ('django.db.models.fields.PositiveIntegerField', [], {}),
            'user': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['auth.User']", 'unique': 'True'})
        },
        'twitter.twitterrequesttoken': {
            'Meta': {'object_name': 'TwitterRequestToken'},
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'oauth_token': ('django.db.models.fields.CharField', [], {'max_length': '80'}),
            'oauth_token_secret': ('django.db.models.fields.CharField', [], {'max_length': '80'}),
            'profile': ('django.db.models.fields.related.OneToOneField', [], {'related_name': "'request_token'", 'unique': 'True', 'to': "orm['twitter.TwitterProfile']"})
        },
        'twitter.twittersettings': {
            'Meta': {'object_name': 'TwitterSettings'},
            'enabled': ('django.db.models.fields.BooleanField', [], {'default': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'post_drink_poured': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'post_session_joined': ('django.db.models.fields.BooleanField', [], {'default': 'True'}),
            'profile': ('django.db.models.fields.related.OneToOneField', [], {'related_name': "'settings'", 'unique': 'True', 'to': "orm['twitter.TwitterProfile']"}),
            'twitter_name': ('django.db.models.fields.CharField', [], {'max_length': '64'})
        }
    }

    complete_apps = ['twitter']


# Copyright 2010 Mike Wakerly <opensource@hoho.com>
#
# This file is part of the Pykeg package of the Kegbot project.
# For more information on Pykeg or Kegbot, see http://kegbot.org/
#
# Pykeg is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#
# Pykeg is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Pykeg.  If not, see <http://www.gnu.org/licenses/>.

"""Chart AJAX views."""

from django.shortcuts import get_object_or_404

from pykeg.core import models
from pykeg.web.api.views import api_view
from pykeg.web.charts import charts

@api_view
def temperature_sensor_history(request, nice_name):
  sensor = get_object_or_404(models.ThermoSensor, nice_name=nice_name,
      site=request.kbsite)
  return charts.TemperatureSensorChart(sensor)

@api_view
def keg_volume(request, keg_id):
  keg = get_object_or_404(models.Keg, seqn=keg_id, site=request.kbsite)
  return charts.KegVolumeChart(keg)

@api_view
def keg_usage_weekdays(request, keg_id):
  keg = get_object_or_404(models.Keg, seqn=keg_id, site=request.kbsite)
  return charts.KegUsageByWeekday(keg)

@api_view
def keg_users_by_volume(request, keg_id):
  keg = get_object_or_404(models.Keg, seqn=keg_id, site=request.kbsite)
  return charts.UsersByVolume(keg)

@api_view
def user_usage_weekdays(request, username):
  user = get_object_or_404(models.User, username=username)
  return charts.UserSessionsByWeekday(user)

@api_view
def user_usage_sessions(request, username):
  user = get_object_or_404(models.User, username=username)
  sessions = user.user_session_chunks.all()
  return charts.SessionVolumes(sessions)

@api_view
def user_session_chunks(request, chunk_id):
  chunk = get_object_or_404(models.UserSessionChunk, site=request.kbsite,
      seqn=chunk_id)
  return charts.UserSessionChunks(chunk)

@api_view
def session_users_by_volume(request, session_id):
  session = get_object_or_404(models.DrinkingSession,
      site=request.kbsite, seqn=session_id)
  sessions = user.user_session_chunks.all()
  return charts.SessionVolumes(sessions)




# Copyright 2012 Mike Wakerly <opensource@hoho.com>
#
# This file is part of the Pykeg package of the Kegbot project.
# For more information on Pykeg or Kegbot, see http://kegbot.org/
#
# Pykeg is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#
# Pykeg is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Pykeg.  If not, see <http://www.gnu.org/licenses/>.

from pykeg.core import models
from pykeg.web.api import util as apiutil

from django.conf import settings
from django.core.urlresolvers import reverse
from django.http import HttpResponse
from django.http import Http404
from django.http import HttpResponseRedirect
from django.shortcuts import get_object_or_404
from django.template.response import SimpleTemplateResponse
from django.template import RequestContext

ALLOWED_PATHS = (
    '/api/login/',
    '/api/get-api-key/',
    '/accounts/login/',
    '/admin/',
    '/media/',
    '/setup/',
)

def _path_allowed(path, kbsite):
  if kbsite:
    path = path.lstrip(kbsite.url())
  for p in ALLOWED_PATHS:
    if path.startswith(p):
      return True
  return False


class KegbotSiteMiddleware:
  ALLOWED_VIEW_MODULE_PREFIXES = (
      'pykeg.web.setup_wizard.',
  )
  def process_view(self, request, view_func, view_args, view_kwargs):
    """Removes kbsite_name from kwargs if present, and attaches the
    corresponding KegbotSite instance to the request as the "kbsite" attribute.

    If kbsite_name is None, the default site is selected.
    """
    kbsite_name = view_kwargs.pop('kbsite_name', None)
    if not kbsite_name:
      kbsite_name = 'default'
    request.kbsite_name = kbsite_name
    try:
      request.kbsite = models.KegbotSite.objects.get(name=kbsite_name)
    except models.KegbotSite.DoesNotExist:
      request.kbsite = None

    if not request.kbsite.is_setup:
      module = view_func.__module__
      allowed = False
      for prefix in self.ALLOWED_VIEW_MODULE_PREFIXES:
        if module.startswith(prefix):
          allowed = True
          break
      if not allowed:
        return SimpleTemplateResponse('setup_wizard/setup_required.html',
            context=RequestContext(request), status=403)
    return None


class SiteActiveMiddleware:
  """Middleware which throws 503s when KegbotSite.is_active is false."""
  def process_view(self, request, view_func, view_args, view_kwargs):
    if not hasattr(request, 'kbsite'):
      return None
    kbsite = request.kbsite

    # We have a KegbotSite, and that site is active: nothing to do.
    if kbsite.is_active:
      return None

    # If the request is for a whitelisted path, allow it.
    if _path_allowed(request.path, kbsite):
      return None

    # Allow staff/superusers access if inactive.
    if request.user.is_staff or request.user.is_superuser:
      return None

    return HttpResponse('Site temporarily unavailable', status=503)


class PrivacyMiddleware:
  """Enforces site privacy settings."""
  def process_view(self, request, view_func, view_args, view_kwargs):
    if not hasattr(request, 'kbsite'):
      return None
    elif _path_allowed(request.path, request.kbsite):
      return None
    elif apiutil.request_is_authenticated(request):
      # This is an auth-required kb api view; no need to check privacy since API
      # keys are given staff-level access.
      return None

    privacy = request.kbsite.settings.privacy
    if privacy == 'public':
      return None

    # If non-public, apply the API key check.
    if apiutil.is_api_view(view_func):
      try:
        apiutil.check_api_key(request)
        return
      except Exception, e:
        return apiutil.wrap_exception(request, e)

    if privacy == 'staff' and not request.user.is_staff:
      return SimpleTemplateResponse('kegweb/staff_only.html',
          context=RequestContext(request), status=401)
    elif privacy == 'members':
      if not request.user.is_authenticated or not request.user.is_active:
        return SimpleTemplateResponse('kegweb/members_only.html',
            context=RequestContext(request), status=401)
      return None

    return HttpResponse('Server misconfigured, unknown privacy setting:%s' % privacy, status=500)




# -*- coding: latin-1 -*-
# Copyright 2009 Mike Wakerly <opensource@hoho.com>
#
# This file is part of the Pykeg package of the Kegbot project.
# For more information on Pykeg or Kegbot, see http://kegbot.org/
#
# Pykeg is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#
# Pykeg is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Pykeg.  If not, see <http://www.gnu.org/licenses/>.

from django.db import models
from django.contrib.auth.models import User
from django.contrib import admin

class Page(models.Model):
  STATUS_CHOICES = (
    ('published', 'published'),
    ('draft', 'draft'),
    ('deleted', 'deleted'),
  )
  MARKUP_CHOICES = (
    ('html', 'html'),
    ('markdown', 'markdown'),
    ('plaintext', 'plaintext'),
  )
  name = models.CharField(max_length=256)
  title = models.CharField(max_length=256)
  author = models.ForeignKey(User)
  content = models.TextField()
  status = models.CharField(max_length=32, choices=STATUS_CHOICES,
    default='published')
  markup = models.CharField(max_length=32, choices=MARKUP_CHOICES,
    default='markdown')
  created_on = models.DateTimeField(editable=False, auto_now_add=True)
  last_modified = models.DateTimeField(auto_now=True)

admin.site.register(Page)



#!/usr/bin/env python
#
# Copyright 2012 Mike Wakerly <opensource@hoho.com>
#
# This file is part of the Pykeg package of the Kegbot project.
# For more information on Pykeg or Kegbot, see http://kegbot.org/
#
# Pykeg is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#
# Pykeg is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Pykeg.  If not, see <http://www.gnu.org/licenses/>.

from django.contrib import messages
from django.core.urlresolvers import reverse
from django.http import HttpResponseRedirect
from socialregistration.clients.oauth import OAuthError
from socialregistration.contrib.twitter.client import Twitter
from django.contrib.admin.views.decorators import staff_member_required

from . import models

class TwitterClient(Twitter):
  def set_callback_url(self, url):
    self.callback_url = url
  def get_callback_url(self):
    return self.callback_url

def _base_url(request):
  """Returns the base absolute URL for this request."""
  host = request.META['HTTP_HOST']
  return 'http://' + host

def _site_url(request, url_name):
  """Shortcut for getting the kbsite-prfixed named URL."""
  kwargs = {
    'kbsite_name': request.kbsite.url(),
  }
  return _base_url(request) + reverse(url_name, kwargs=kwargs)

def _site_twitter_session_key(request):
  """Returns a site-specific session key."""
  return 'site_twitter_%s' % (request.kbsite.id,)

@staff_member_required
def site_twitter_redirect(request):
  if request.POST.get('remove') is not None:
    request.kbsite.twitter_profile.delete()
    url = _site_url(request, 'kegadmin-connections')
    messages.success(request, 'Removed Twitter account.')
    return HttpResponseRedirect(url)

  client = TwitterClient()
  url = _site_url(request, 'site_twitter_callback')
  client.set_callback_url(url)
  request.session[_site_twitter_session_key(request)] = client

  try:
    return HttpResponseRedirect(client.get_redirect_url())
  except OAuthError, error:
    url = _site_url(request, 'kegadmin-connections')
    messages.error(request, 'Error: %s' % str(error))
    return HttpResponseRedirect(url)

@staff_member_required
def site_twitter_callback(request):
  token = None
  try:
    client = request.session[_site_twitter_session_key(request)]
    token = client.complete(dict(request.GET.items()))
  except KeyError:
    messages.error(request, 'Session expired.')
  except OAuthError, error:
    messages.error(request, str(error))
  else:
    site = request.kbsite
    user_info = client.get_user_info()

    try:
      profile = models.SiteTwitterProfile.objects.get(site=site)
    except models.SiteTwitterProfile.DoesNotExist:
      profile = models.SiteTwitterProfile(site=site)
    profile.oauth_token = token.key
    profile.oauth_token_secret = token.secret
    profile.twitter_name = user_info['screen_name']
    profile.twitter_id = int(user_info['user_id'])
    profile.enabled = True
    profile.save()
    messages.success(request, 'Successfully linked to @%s' % user_info['screen_name'])

  url = _site_url(request, 'kegadmin-connections')
  return HttpResponseRedirect(url)



# Copyright 2010 Mike Wakerly <opensource@hoho.com>
#
# This file is part of the Pykeg package of the Kegbot project.
# For more information on Pykeg or Kegbot, see http://kegbot.org/
#
# Pykeg is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#
# Pykeg is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Pykeg.  If not, see <http://www.gnu.org/licenses/>.

from django.conf.urls.defaults import *

urlpatterns = patterns('pykeg.web.charts.views',
    url(r'^temperature-sensor/(?P<nice_name>\w+)/history/?$',
        'temperature_sensor_history'),
    url(r'^keg/(?P<keg_id>\d+)/volume/?$', 'keg_volume'),
    url(r'^keg/(?P<keg_id>\d+)/usage-by-weekday/?$', 'keg_usage_weekdays'),
    url(r'^keg/(?P<keg_id>\d+)/users-by-volume/?$', 'keg_users_by_volume'),
    url(r'^user/(?P<username>[\w@.+-_]+)/usage-by-weekday/?$', 'user_usage_weekdays'),
    url(r'^user/(?P<username>[\w@.+-_]+)/usage-by-session/?$', 'user_usage_sessions'),
    url(r'^session/(?P<session_id>\d+)/users-by-volume/?$',
        'session_users_by_volume'),
)



"""Unittests for pykeg.core.models"""

import datetime
import unittest

from pykeg.core.backend.django import KegbotBackend
from pykeg.core import kb_common
from pykeg.core import models

from kegbot.util import units

from pykeg.beerdb import models as bdb_models

class CoreModelsTestCase(unittest.TestCase):
  def setUp(self):
    models.KegbotSite.objects.filter(name='default').delete()
    self.site, created = models.KegbotSite.objects.get_or_create(name='default')
    self.backend = KegbotBackend(site=self.site)
    self.brewer = bdb_models.Brewer.objects.create(
        name='Moonshine Beers',
        country='USA',
        origin_state='Anystate',
        origin_city='Bathtub',
        production='retail',
        url='http://example.com/',
        description='Pretty bad beers.',
    )

    self.beer_style = bdb_models.BeerStyle.objects.create(
        name='Porter',
    )

    self.beer_type = bdb_models.BeerType.objects.create(
        name='Moonshine Porter',
        brewer=self.brewer,
        style=self.beer_style,
        calories_oz=3.0,
        carbs_oz=10.0,
        abv=0.05,
    )

    self.keg_vol = units.UnitConverter.Convert(2.0, units.UNITS.Liter,
                                               units.RECORD_UNIT)
    self.keg_size = models.KegSize.objects.create(
        name='Tiny Keg',
        volume_ml=self.keg_vol,
    )

    self.keg = models.Keg.objects.create(
        site=self.site,
        type=self.beer_type,
        size=self.keg_size,
        start_time=datetime.datetime(2000, 4, 1),
        end_time=datetime.datetime(2000, 5, 1),
        status='online',
        description='Our first keg!',
        origcost=99.0,
    )

    self.tap = models.KegTap.objects.create(
        site=self.site,
        name='Test Tap',
        meter_name='test',
        ml_per_tick=(1000.0/2200.0),
        current_keg=self.keg,
    )

    self.user = models.User.objects.create(
        username='kb_tester',
    )

    self.user2 = models.User.objects.create(
        username='kb_tester2',
    )

  def tearDown(self):
    self.user.delete()
    self.user2.delete()
    self.keg.delete()
    del self.keg_vol
    self.beer_type.delete()
    self.beer_style.delete()
    self.brewer.delete()

  def testKegStuff(self):
    """Test basic keg relations that should always work."""
    self.assertEqual(self.keg.size.volume_ml,
        units.Quantity(2.0, units.UNITS.Liter).InMilliliters())
    self.assertEqual(self.keg.type.brewer.name, "Moonshine Beers")

    self.assertEqual(self.keg.served_volume(), 0.0)
    self.assertEqual(self.keg.remaining_volume(), self.keg_vol)

  def testDrinkAccounting(self):
    vol = units.Quantity(1200)

    d = self.backend.RecordDrink(tap_name=self.tap.meter_name,
        ticks=1200,
        username=self.user.username,
    )

    print d

    self.assertEqual(self.keg.served_volume(), d.volume_ml)

  def testDrinkSessions(self):
    """ Checks for the DrinkingSession records. """
    u1 = self.user
    u2 = self.user2
    vol = units.Quantity(1200)

    drinks = {}
    base_time = datetime.datetime(2009,1,1,1,0,0)

    ticks = volume = vol.InKbMeterTicks()

    td_10m = datetime.timedelta(minutes=10)
    td_200m = datetime.timedelta(minutes=200)
    td_190m = td_200m - td_10m

    self.assertEqual(models.Drink.objects.all().count(), 0)
    self.assertEqual(models.DrinkingSession.objects.all().count(), 0)

    ### User 1
    # t=0
    print self.backend.RecordDrink(tap_name=self.tap.meter_name,
        ticks=1200,
        username=u1.username,
        pour_time=base_time,
    )
    # t=10
    print self.backend.RecordDrink(tap_name=self.tap.meter_name,
        ticks=1200,
        username=u1.username,
        pour_time=base_time+td_10m,
    )
    # t=200
    print self.backend.RecordDrink(tap_name=self.tap.meter_name,
        ticks=1200,
        username=u1.username,
        pour_time=base_time+td_200m,
    )
    drinks_u1 = u1.drinks.all().order_by('time')


    ### User 2
    # t=0
    print self.backend.RecordDrink(tap_name=self.tap.meter_name,
        ticks=1200,
        username=u2.username,
        pour_time=base_time,
    )
    # t=200
    print self.backend.RecordDrink(tap_name=self.tap.meter_name,
        ticks=1200,
        username=u2.username,
        pour_time=base_time+td_200m,
    )
    # t=190
    print self.backend.RecordDrink(tap_name=self.tap.meter_name,
        ticks=1200,
        username=u2.username,
        pour_time=base_time+td_190m,
    )
    drinks_u2 = u2.drinks.all().order_by('time')

    u1_chunks = u1.session_chunks.all().order_by('start_time')
    self.assertEqual(len(u1_chunks), 2)

    u2_chunks = u2.session_chunks.all().order_by('start_time')
    self.assertEqual(len(u2_chunks), 2)

    s1, s2 = models.DrinkingSession.objects.all().order_by('start_time')[:2]

    SESSION_DELTA = datetime.timedelta(minutes=kb_common.DRINK_SESSION_TIME_MINUTES)

    # session 1: should be 10 minutes long as created above
    self.assertEqual(s1.start_time, drinks_u1[0].start_time)
    self.assertEqual(s1.start_time, drinks_u1[1].end_time)
    self.assertEqual(s1.drinks.valid().filter(user=u1).count(), 2)
    self.assertEqual(s1.drinks.valid().filter(user=u2).count(), 1)

    # session 2: at time 200, 1 drink
    self.assertEqual(s2.start_time, base_time + td_190m)
    self.assertEqual(s2.end_time, base_time + td_200m)
    self.assertEqual(s2.drinks.valid().filter(user=u1).count(), 1)
    self.assertEqual(s2.drinks.valid().filter(user=u2).count(), 2)

    # user2 session2: drinks are added out of order to create this, ensure times
    # match
    u2_c2 = u2_chunks[1]
    self.assertEqual(u2_c2.start_time, base_time+td_190m)
    self.assertEqual(u2_c2.end_time, base_time+td_200m)

    # Now check DrinkingSessions were created correctly; there should be
    # two groups capturing all 4 sessions.
    all_groups = models.DrinkingSession.objects.all().order_by('start_time')
    self.assertEqual(len(all_groups), 2)

    self.assertEqual(all_groups[0].start_time, base_time)
    self.assertEqual(all_groups[0].end_time, base_time+td_10m)
    self.assertEqual(all_groups[0].user_chunks.all().count(), 2)

    self.assertEqual(all_groups[1].start_time, base_time+td_190m)
    self.assertEqual(all_groups[1].end_time, base_time+td_200m)
    self.assertEqual(all_groups[1].user_chunks.all().count(), 2)



# Copyright 2012 Mike Wakerly <opensource@hoho.com>
#
# This file is part of the Pykeg package of the Kegbot project.
# For more information on Pykeg or Kegbot, see http://kegbot.org/
#
# Pykeg is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#
# Pykeg is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Pykeg.  If not, see <http://www.gnu.org/licenses/>.

from django.db import models
from pykeg.core import models as core_models

### Helpers

class _SettingsMixin(models.Model):
  """Common settings for all social accounts."""
  class Meta:
    abstract = True

  enabled = models.BooleanField(default=True,
      help_text='Deselect to disable all posting from this account.')
  post_session_joined = models.BooleanField(default=True,
      help_text='Tweet at the start of a new drinking session.')
  post_drink_poured = models.BooleanField(default=False,
      help_text='Tweet when a new drink is poured.')

### Site-specific

class SiteTwitterProfile(models.Model):
  site = models.OneToOneField(core_models.KegbotSite, related_name='twitter_profile')
  twitter_name = models.CharField(max_length=32)
  twitter_id = models.PositiveIntegerField()
  oauth_token = models.CharField(max_length=80)
  oauth_token_secret = models.CharField(max_length=80)
  enabled = models.BooleanField(default=False)

  def is_enabled(self):
    disabled = (not self.enabled or not self.twitter_name or not self.twitter_id or
      not self.oauth_token or not self.oauth_token_secret)
    return not disabled

class SiteTwitterSettings(_SettingsMixin):
  profile = models.OneToOneField(SiteTwitterProfile, related_name='settings')
  post_unauthenticated = models.BooleanField(default=True,
      help_text='Whether to generate tweets for unauthenticated '
          '(guest) pours.')
  post_unlinked = models.BooleanField(default=True,
      help_text='Whether to generate tweets for authenticated pours '
          'with no linked Twitter account.  The tweet will include the '
          'Kegbot username of the user.')

### User-specific
from socialregistration import signals as sr_signals
from socialregistration.contrib.twitter.models import TwitterProfile

class TwitterSettings(_SettingsMixin):
  profile = models.OneToOneField(TwitterProfile, unique=True,
      related_name='settings')
  twitter_name = models.CharField(max_length=64,
      help_text='Stores the twitter name of this user.')

def _twitter_connect_handler(sender, user, profile, client, request=None, **kwargs):
  settings, _ = TwitterSettings.objects.get_or_create(profile=profile)
  settings.twitter_name = client.get_user_info()['screen_name']
  settings.save()

sr_signals.connect.connect(_twitter_connect_handler, sender=TwitterProfile,
    dispatch_uid = 'twitter.connect')




# Copyright 2012 Mike Wakerly <opensource@hoho.com>
#
# This file is part of the Pykeg package of the Kegbot project.
# For more information on Pykeg or Kegbot, see http://kegbot.org/
#
# Pykeg is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#
# Pykeg is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Pykeg.  If not, see <http://www.gnu.org/licenses/>.

"""Utilities for processing API views."""

from django.conf import settings
from django.http import Http404
from django.http import HttpResponse

from kegbot.api import kbapi
from kegbot.util import kbjson
from pykeg.core import models
from pykeg.core.backend import backend
from . import apikey

import logging
import sys
import traceback

LOGGER = logging.getLogger(__name__)

ATTR_API_VIEW = 'api_view'
ATTR_API_REQUEST = 'api_request'
ATTR_API_AUTHENTICATED = 'api_authenticated'
ATTR_API_AUTH_REQUIRED = 'api_auth_required'

def is_api_view(view):
  return getattr(view, ATTR_API_VIEW, False)

def set_is_api_view(view):
  setattr(view, ATTR_API_VIEW, True)

def is_api_request(request):
  return getattr(request, ATTR_API_REQUEST, False)

def set_is_api_request(request):
  setattr(request, ATTR_API_REQUEST, True)

def request_is_authenticated(request):
  return getattr(request, ATTR_API_AUTHENTICATED, False)

def set_request_is_authenticated(request):
  setattr(request, ATTR_API_AUTHENTICATED, True)

def view_requires_authentication(view):
  return getattr(view, ATTR_API_AUTH_REQUIRED, False)

def set_view_requires_authentication(view):
  setattr(view, ATTR_API_AUTH_REQUIRED, True)

def check_api_key(request):
  """Check a request for an API key."""
  keystr = request.META.get('HTTP_X_KEGBOT_API_KEY')
  if not keystr:
    keystr = request.REQUEST.get('api_key')
  if not keystr:
    raise kbapi.NoAuthTokenError('The parameter "api_key" is required')

  try:
    key = apikey.ApiKey.FromString(keystr)
  except ValueError, e:
    raise kbapi.BadApiKeyError('Error parsing API key: %s' % e)

  try:
    user = models.User.objects.get(pk=key.uid())
  except models.User.DoesNotExist:
    raise kbapi.BadApiKeyError('API user %s does not exist' % key.uid())

  if not user.is_active:
    raise kbapi.BadApiKeyError('User is inactive')

  if not user.is_staff and not user.is_superuser:
    raise kbapi.PermissionDeniedError('User is not staff/superuser')

  user_secret = user.get_profile().api_secret
  if not user_secret or user_secret != key.secret():
    raise kbapi.BadApiKeyError('User secret does not match')

  setattr(request, ATTR_API_AUTHENTICATED, True)


def to_json_error(e, exc_info):
  """Converts an exception to an API error response."""
  # Wrap some common exception types into kbapi types
  if isinstance(e, Http404):
    e = kbapi.NotFoundError(e.message)
  elif isinstance(e, ValueError):
    e = kbapi.BadRequestError(str(e))
  elif isinstance(e, backend.NoTokenError):
    e = kbapi.NotFoundError(e.message)

  # Now determine the response based on the exception type.
  if isinstance(e, kbapi.Error):
    code = e.__class__.__name__
    http_code = e.HTTP_CODE
    message = e.Message()
  else:
    code = 'ServerError'
    http_code = 500
    message = 'An internal error occurred: %s' % str(e)
    if settings.DEBUG:
      message += "\n" + "\n".join(traceback.format_exception(*exc_info))
  result = {
    'error' : {
      'code' : code,
      'message' : message
    }
  }
  return result, http_code

def build_response(result_data, response_code=200):
  """Builds an HTTP response for JSON data."""
  indent = 2
  return HttpResponse(kbjson.dumps(result_data, indent=indent),
      mimetype='application/json', status=response_code)

def wrap_exception(request, exception):
  """Returns a HttpResponse with the exception in JSON form."""
  exc_info = sys.exc_info()

  LOGGER.error('%s: %s' % (exception.__class__.__name__, exception),
      exc_info=exc_info,
      extra={
        'status_code': 500,
        'request': request,
      }
  )

  if settings.DEBUG and settings.HAVE_RAVEN:
    from raven.contrib.django.models import client
    client.captureException()

  # Don't wrap the exception during debugging.
  if settings.DEBUG and 'deb' in request.GET:
    return None

  result_data, http_code = to_json_error(exception, exc_info)
  result_data['meta'] = {
    'result': 'error'
  }
  return build_response(result_data, response_code=http_code)





# Copyright 2012 Mike Wakerly <opensource@hoho.com>
#
# This file is part of the Pykeg package of the Kegbot project.
# For more information on Pykeg or Kegbot, see http://kegbot.org/
#
# Pykeg is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#
# Pykeg is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Pykeg.  If not, see <http://www.gnu.org/licenses/>.

from django.conf import settings
from . import util
import logging
import sys

LOGGER = logging.getLogger(__name__)

def wrap_exception(request, exception):
  """Returns a HttpResponse with the exception in JSON form."""
  exc_info = sys.exc_info()

  LOGGER.error('%s: %s' % (exception.__class__.__name__, exception),
      exc_info=exc_info,
      extra={
        'status_code': 500,
        'request': request,
      }
  )

  if settings.DEBUG and settings.HAVE_RAVEN:
    from raven.contrib.django.models import client
    client.captureException()

  # Don't wrap the exception during debugging.
  if settings.DEBUG and 'deb' in request.GET:
    return None

  result_data, http_code = util.to_json_error(exception, exc_info)
  result_data['meta'] = {
    'result': 'error'
  }
  return util.build_response(result_data, response_code=http_code)


class WrapExceptionMiddleware:
  """Translates responses and exceptions to JSON."""
  def process_view(self, request, view_func, view_args, view_kwargs):
    if util.is_api_view(view_func):
      util.set_is_api_request(request)

  def process_exception(self, request, exception):
    if not util.is_api_request(request):
      return None
    return wrap_exception(request, exception)


class CheckAccessMiddleware:
  def process_view(self, request, view_func, view_args, view_kwargs):
    if not util.is_api_view(view_func):
      return None

    if not util.request_is_authenticated(request):
      if util.view_requires_authentication(view_func):
        try:
          util.check_api_key(request)
        except Exception, e:
          return wrap_exception(request, e)



"""Defines HAVE_* flags for several optional packages."""

try:
  import debug_toolbar
  HAVE_DEBUG_TOOLBAR = True
except ImportError:
  HAVE_DEBUG_TOOLBAR = False

try:
  import raven.contrib.django
  HAVE_RAVEN = True
except ImportError:
  HAVE_RAVEN = False

try:
  import sentry
  HAVE_SENTRY = True
except ImportError:
  HAVE_SENTRY = False

try:
  import celery
  HAVE_CELERY = True
except ImportError:
  HAVE_CELERY = False

try:
  import djcelery
  HAVE_DJCELERY = True
except ImportError:
  HAVE_DJCELERY = False

try:
  import djkombu
  HAVE_DJKOMBU = True
except ImportError:
  HAVE_DJKOMBU = False

try:
  import rjdj.djangotornado
  HAVE_DJANGOTORNADO = True
except ImportError:
  HAVE_DJANGOTORNADO = False



# encoding: utf-8
import datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models

class Migration(SchemaMigration):

    def forwards(self, orm):
        db.rename_column('beerdb_beerimage', 'original_image', 'image')

    def backwards(self, orm):
        db.rename_column('beerdb_beerimage', 'image', 'original_image')

    models = {
        'beerdb.beerimage': {
            'Meta': {'object_name': 'BeerImage'},
            'added': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            'edited': ('django.db.models.fields.DateTimeField', [], {}),
            'id': ('django.db.models.fields.CharField', [], {'max_length': '36', 'primary_key': 'True'}),
            'image': ('django.db.models.fields.files.ImageField', [], {'max_length': '100'}),
            'num_views': ('django.db.models.fields.PositiveIntegerField', [], {'default': '0'}),
            'revision': ('django.db.models.fields.IntegerField', [], {'default': '0'})
        },
        'beerdb.beerstyle': {
            'Meta': {'object_name': 'BeerStyle'},
            'added': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            'edited': ('django.db.models.fields.DateTimeField', [], {}),
            'id': ('django.db.models.fields.CharField', [], {'max_length': '36', 'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '128'}),
            'revision': ('django.db.models.fields.IntegerField', [], {'default': '0'})
        },
        'beerdb.beertype': {
            'Meta': {'object_name': 'BeerType'},
            'abv': ('django.db.models.fields.FloatField', [], {'null': 'True', 'blank': 'True'}),
            'added': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            'brewer': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['beerdb.Brewer']"}),
            'calories_oz': ('django.db.models.fields.FloatField', [], {'null': 'True', 'blank': 'True'}),
            'carbs_oz': ('django.db.models.fields.FloatField', [], {'null': 'True', 'blank': 'True'}),
            'edited': ('django.db.models.fields.DateTimeField', [], {}),
            'edition': ('django.db.models.fields.CharField', [], {'max_length': '255', 'null': 'True', 'blank': 'True'}),
            'id': ('django.db.models.fields.CharField', [], {'max_length': '36', 'primary_key': 'True'}),
            'image': ('django.db.models.fields.related.ForeignKey', [], {'blank': 'True', 'related_name': "'beers'", 'null': 'True', 'to': "orm['beerdb.BeerImage']"}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '255'}),
            'original_gravity': ('django.db.models.fields.FloatField', [], {'null': 'True', 'blank': 'True'}),
            'revision': ('django.db.models.fields.IntegerField', [], {'default': '0'}),
            'specific_gravity': ('django.db.models.fields.FloatField', [], {'null': 'True', 'blank': 'True'}),
            'style': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['beerdb.BeerStyle']"}),
            'untappd_beer_id': ('django.db.models.fields.IntegerField', [], {'null': 'True', 'blank': 'True'})
        },
        'beerdb.brewer': {
            'Meta': {'object_name': 'Brewer'},
            'added': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            'country': ('pykeg.core.fields.CountryField', [], {'default': "'USA'", 'max_length': '3'}),
            'description': ('django.db.models.fields.TextField', [], {'default': "''", 'null': 'True', 'blank': 'True'}),
            'edited': ('django.db.models.fields.DateTimeField', [], {}),
            'id': ('django.db.models.fields.CharField', [], {'max_length': '36', 'primary_key': 'True'}),
            'image': ('django.db.models.fields.related.ForeignKey', [], {'blank': 'True', 'related_name': "'brewers'", 'null': 'True', 'to': "orm['beerdb.BeerImage']"}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '255'}),
            'origin_city': ('django.db.models.fields.CharField', [], {'default': "''", 'max_length': '128', 'null': 'True', 'blank': 'True'}),
            'origin_state': ('django.db.models.fields.CharField', [], {'default': "''", 'max_length': '128', 'null': 'True', 'blank': 'True'}),
            'production': ('django.db.models.fields.CharField', [], {'default': "'commercial'", 'max_length': '128'}),
            'revision': ('django.db.models.fields.IntegerField', [], {'default': '0'}),
            'url': ('django.db.models.fields.URLField', [], {'default': "''", 'max_length': '200', 'null': 'True', 'blank': 'True'})
        }
    }

    complete_apps = ['beerdb']



# Copyright 2010 Mike Wakerly <opensource@hoho.com>
#
# This file is part of the Pykeg package of the Kegbot project.
# For more information on Pykeg or Kegbot, see http://kegbot.org/
#
# Pykeg is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#
# Pykeg is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Pykeg.  If not, see <http://www.gnu.org/licenses/>.

from django.core.management.base import CommandError
from django.core.management.base import NoArgsCommand

from pykeg.core import models
from pykeg.core.management.commands.common import progbar


class Command(NoArgsCommand):
  help = u'Regenerate all cached stats.'
  args = '<none>'

  def handle(self, **options):
    models.SystemStats.objects.all().delete()
    models.KegStats.objects.all().delete()
    models.UserStats.objects.all().delete()
    models.SessionStats.objects.all().delete()

    for site in models.KegbotSite.objects.all():
      print 'site: %s' % site
      self.handle_site(site)

  def handle_site(self, site):
    drinks = site.drinks.valid()

    last_drinks = drinks.order_by('-time')
    if last_drinks:
      progbar('recalc system stats', 0, 1)
      last_drinks[0]._UpdateSystemStats()
    progbar('recalc system stats', 1, 1)
    print ''

    kegs = site.kegs.all()
    count = kegs.count()
    pos = 0
    for k in kegs:
      pos += 1
      progbar('recalc keg stats', pos, count)
      last_drinks = k.drinks.valid().order_by('-time')
      if last_drinks:
        last_drinks[0]._UpdateKegStats()
    print ''

    users = models.User.objects.all()
    count = users.count()
    pos = 0
    for user in users:
      pos += 1
      progbar('recalc user stats', pos, count)
      user_drinks = user.drinks.valid().filter(site=site).order_by('-time')
      if user_drinks:
        last = user_drinks[0]
        last._UpdateUserStats()
    print ''

    sessions = models.DrinkingSession.objects.all()
    count = sessions.count()
    pos = 0
    for session in sessions:
      pos += 1
      progbar('recalc session stats', pos, count)
      session_drinks = session.drinks.valid().filter(site=site).order_by('-time')
      if session_drinks:
        last = session_drinks[0]
        last._UpdateSessionStats()
    print ''

    print 'done!'



#!/usr/bin/env python

"""Kegbot Beer Kegerator Server package.

Kegbot is a hardware and software system to record and monitor access to a beer
kegerator.  For more information and documentation, see http://kegbot.org/
"""

from setuptools import setup, find_packages
from pykeg import __version__ as VERSION

DOCLINES = __doc__.split('\n')

SHORT_DESCRIPTION = DOCLINES[0]
LONG_DESCRIPTION = '\n'.join(DOCLINES[2:])
DEPENDENCIES = [
  'kegbot-pyutils >= 0.1.4',
  'kegbot-api >= 0.1.6',

  'django >= 1.3',
  'django-autoslug',
  'django-imagekit >= 2.0',
  'django-registration',
  'django-socialregistration >= 0.5.4',
  'django_extensions',

  'Celery',
  'django-celery',
  'django-kombu',

  'South >= 0.7.3',
  'django-crispy-forms',
  'django-icanhaz',
  'django_nose',
  'facebook-sdk >= 0.3.0',
  'foursquare',
  'gunicorn >= 0.16.1',
  'poster', # needed by foursquare
  'pylcdui >= 0.5.5',
  'protobuf >= 2.4.1',
  'python-gflags >= 1.8',
  'pytz',
  'requests', # needed by oauth
  'tweepy',
]

def setup_package():
  setup(
      name = 'kegbot',
      version = VERSION,
      description = SHORT_DESCRIPTION,
      long_description = LONG_DESCRIPTION,
      author = 'mike wakerly',
      author_email = 'opensource@hoho.com',
      url = 'http://kegbot.org/',
      packages = find_packages(),
      scripts = [
        'bin/kegbot-admin.py',
        'bin/setup-kegbot.py',
      ],
      install_requires = DEPENDENCIES,
      dependency_links = [
          'https://github.com/rem/python-protobuf/tarball/master#egg=protobuf-2.4.1',
      ],
      include_package_data = True,
      entry_points = {
        'console_scripts': ['instance=django.core.management:execute_manager'],
      },
  )

if __name__ == '__main__':
  setup_package()



from django.conf.urls.defaults import *

urlpatterns = patterns('pykeg.web.kegweb.views',
      ### main page
      url(r'^$', 'index', name='kb-home'),

      ### accountpage
      url(r'^claim_token/$', 'claim_token', name='kb-claim-token'),

      ### all-time stats
      url(r'^stats/$', 'system_stats', name='kb-stats'),
      url(r'^leaders/$', 'redirect_to', {'url': '/stats/'}),

      ### keg related
      url(r'^kegs/$', 'keg_list', name='kb-kegs'),
      url(r'^kegs/(?P<keg_id>\d+)', 'keg_detail', name='kb-keg'),
      # redirects to the above for compatibility
      (r'^keg/(?P<keg_id>\d+)', 'redirect_to', {'url': '/kegs/%(keg_id)s'}),

      ### drinkers
      url(r'^drinkers/$', 'user_list', name='kb-drinkers'),
      url(r'^drinkers/(?P<username>[\w@.+-_]+)/$', 'user_detail', name='kb-drinker'),
      url(r'^drinkers/(?P<user_id>\d+)/$', 'user_detail_by_id'),
      # redirects to the above for compatibility
      (r'^drinker/(?P<user_id>\d+)/$', 'redirect_to', {'url': '/drinkers/%(user_id)s'}),
      (r'^drinker/(?P<username>[\w@.+-_]+)/$', 'redirect_to', {'url': '/drinkers/%(username)s'}),

      ### drink related
      url(r'^drinks/(?P<drink_id>\d+)', 'drink_detail', name='kb-drink'),
      # redirects to the above for compatibility
      (r'^drink/(?P<drink_id>\d+)', 'short_drink_detail'),
      (r'^d/(?P<drink_id>\d+)', 'short_drink_detail'),

      ### sessions
      url(r'^session/(?P<year>\d{4})-(?P<month>\d{1,2})-(?P<day>\d{1,2})/(?P<seqn>\d+)/(?P<slug>[-\w]+)',
          'session_detail', name='kb-session'),

      (r'^session/(?P<session_id>\d+)/$', 'short_session_detail'),
      (r'^s/(?P<session_id>\d+)/$', 'short_session_detail'),

)




from django.conf import settings
from django.conf.urls.defaults import *

if settings.DEBUG:
  urlpatterns = patterns('pykeg.web.setup_wizard.views',
      url(r'^$', 'start', name='setup_wizard_start'),
      url(r'^settings/$', 'site_settings', name='setup_site_settings'),
      url(r'^admin/$', 'admin', name='setup_admin'),
      url(r'^finish/$', 'finish', name='setup_finish'),
  )
else:
  # For safety, disable setup when not in DEBUG mode.
  urlpatterns = patterns('')



# -*- coding: latin-1 -*-

from south.db import db
from django.db import models
from pykeg.core.models import *

class Migration:
    
    def forwards(self, orm):
        
        # Deleting field 'Keg.channel'
        db.delete_column('core_keg', 'channel')
        
    
    
    def backwards(self, orm):
        
        # Adding field 'Keg.channel'
        db.add_column('core_keg', 'channel', orm['core.keg:channel'])
        
    
    
    models = {
        'auth.group': {
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '80'}),
            'permissions': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['auth.Permission']", 'blank': 'True'})
        },
        'auth.permission': {
            'Meta': {'unique_together': "(('content_type', 'codename'),)"},
            'codename': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'content_type': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['contenttypes.ContentType']"}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '50'})
        },
        'auth.user': {
            'date_joined': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            'email': ('django.db.models.fields.EmailField', [], {'max_length': '75', 'blank': 'True'}),
            'first_name': ('django.db.models.fields.CharField', [], {'max_length': '30', 'blank': 'True'}),
            'groups': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['auth.Group']", 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'is_active': ('django.db.models.fields.BooleanField', [], {'default': 'True', 'blank': 'True'}),
            'is_staff': ('django.db.models.fields.BooleanField', [], {'default': 'False', 'blank': 'True'}),
            'is_superuser': ('django.db.models.fields.BooleanField', [], {'default': 'False', 'blank': 'True'}),
            'last_login': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            'last_name': ('django.db.models.fields.CharField', [], {'max_length': '30', 'blank': 'True'}),
            'password': ('django.db.models.fields.CharField', [], {'max_length': '128'}),
            'user_permissions': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['auth.Permission']", 'blank': 'True'}),
            'username': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '30'})
        },
        'contenttypes.contenttype': {
            'Meta': {'unique_together': "(('app_label', 'model'),)", 'db_table': "'django_content_type'"},
            'app_label': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'model': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '100'})
        },
        'core.bac': {
            'bac': ('django.db.models.fields.FloatField', [], {}),
            'drink': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['core.Drink']"}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'rectime': ('django.db.models.fields.DateTimeField', [], {}),
            'user': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['auth.User']"})
        },
        'core.beerstyle': {
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '128'})
        },
        'core.beertype': {
            'abv': ('django.db.models.fields.FloatField', [], {'default': '0'}),
            'brewer': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['core.Brewer']"}),
            'calories_oz': ('django.db.models.fields.FloatField', [], {'default': '0'}),
            'carbs_oz': ('django.db.models.fields.FloatField', [], {'default': '0'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '128'}),
            'style': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['core.BeerStyle']"})
        },
        'core.brewer': {
            'comment': ('django.db.models.fields.TextField', [], {'default': "''", 'null': 'True', 'blank': 'True'}),
            'distribution': ('django.db.models.fields.CharField', [], {'default': "'unknown'", 'max_length': '128'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '128'}),
            'origin_city': ('django.db.models.fields.CharField', [], {'default': "''", 'max_length': '128', 'null': 'True', 'blank': 'True'}),
            'origin_country': ('django.db.models.fields.CharField', [], {'default': "''", 'max_length': '128'}),
            'origin_state': ('django.db.models.fields.CharField', [], {'default': "''", 'max_length': '128'}),
            'url': ('django.db.models.fields.URLField', [], {'default': "''", 'max_length': '200', 'null': 'True', 'blank': 'True'})
        },
        'core.config': {
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'key': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '256'}),
            'value': ('django.db.models.fields.TextField', [], {})
        },
        'core.drink': {
            'endtime': ('django.db.models.fields.DateTimeField', [], {}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'keg': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['core.Keg']", 'null': 'True', 'blank': 'True'}),
            'starttime': ('django.db.models.fields.DateTimeField', [], {}),
            'status': ('django.db.models.fields.CharField', [], {'default': "'valid'", 'max_length': '128'}),
            'ticks': ('django.db.models.fields.PositiveIntegerField', [], {}),
            'user': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['auth.User']"}),
            'volume': ('django.db.models.fields.PositiveIntegerField', [], {})
        },
        'core.drinkingsessiongroup': {
            'endtime': ('django.db.models.fields.DateTimeField', [], {}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'starttime': ('django.db.models.fields.DateTimeField', [], {})
        },
        'core.keg': {
            'description': ('django.db.models.fields.CharField', [], {'max_length': '256'}),
            'enddate': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'origcost': ('django.db.models.fields.FloatField', [], {'default': '0'}),
            'size': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['core.KegSize']"}),
            'startdate': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            'status': ('django.db.models.fields.CharField', [], {'max_length': '128'}),
            'type': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['core.BeerType']"})
        },
        'core.kegsize': {
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '128'}),
            'volume': ('django.db.models.fields.IntegerField', [], {})
        },
        'core.kegtap': {
            'current_keg': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['core.Keg']", 'null': 'True', 'blank': 'True'}),
            'description': ('django.db.models.fields.TextField', [], {'null': 'True', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'meter_name': ('django.db.models.fields.CharField', [], {'max_length': '128'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '128'})
        },
        'core.relaylog': {
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '128'}),
            'status': ('django.db.models.fields.CharField', [], {'max_length': '32'}),
            'time': ('django.db.models.fields.DateTimeField', [], {})
        },
        'core.thermolog': {
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '128'}),
            'temp': ('django.db.models.fields.FloatField', [], {}),
            'time': ('django.db.models.fields.DateTimeField', [], {})
        },
        'core.token': {
            'created': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'keyinfo': ('django.db.models.fields.TextField', [], {}),
            'user': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['auth.User']"})
        },
        'core.userdrinkingsession': {
            'endtime': ('django.db.models.fields.DateTimeField', [], {}),
            'group': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['core.DrinkingSessionGroup']"}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'starttime': ('django.db.models.fields.DateTimeField', [], {}),
            'user': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['auth.User']"})
        },
        'core.userdrinkingsessionassignment': {
            'drink': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['core.Drink']"}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'session': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['core.UserDrinkingSession']"})
        },
        'core.userlabel': {
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'labelname': ('django.db.models.fields.CharField', [], {'max_length': '128'})
        },
        'core.userpicture': {
            'active': ('django.db.models.fields.BooleanField', [], {'default': 'True', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'image': ('django.db.models.fields.files.ImageField', [], {'max_length': '100'}),
            'user': ('django.db.models.fields.related.OneToOneField', [], {'to': "orm['auth.User']", 'unique': 'True'})
        },
        'core.userprofile': {
            'gender': ('django.db.models.fields.CharField', [], {'max_length': '8'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'labels': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['core.UserLabel']"}),
            'user': ('django.db.models.fields.related.OneToOneField', [], {'to': "orm['auth.User']", 'unique': 'True'}),
            'weight': ('django.db.models.fields.FloatField', [], {})
        }
    }
    
    complete_apps = ['core']



from django.conf import settings
from socialregistration.contrib.twitter import models as sr_twitter_models
import tweepy

from . import models

_CONSUMER_KEY = getattr(settings, 'TWITTER_CONSUMER_KEY', '')
_CONSUMER_SECRET = getattr(settings, 'TWITTER_CONSUMER_SECRET_KEY', '')

HEADERS = {
  'User-Agent': 'Kegbot',
}

# Available template vars:
#   name         : user name, or @twitter_name if known
#   kb_name      : kegbot system name
#   kb_url       : kegbot system url
#   session_url  : current session url
#   drink_url    : drink url (if any)
#   drink_size   : drink size
#   beer_name    : beer name

DEFAULT_USER_SESSION_JOINED_TEMPLATE = "I'm having a drink on %(kb_name)s! %(kb_url)s"
DEFAULT_USER_DRINK_POURED_TEMPLATE = "I just poured %(drink_size)s of %(beer_name)s on %(kb_name)s! %(drink_url)s"

DEFAULT_SYSTEM_SESSION_JOINED_TEMPLATE = "%(name)s is having a drink on %(kb_name)s! %(kb_url)s"
DEFAULT_SYSTEM_DRINK_POURED_TEMPLATE = "%(name)s just poured %(drink_size)s of %(beer_name)s on %(kb_name)s! %(drink_url)s"

# Appended if space is permitting.
HASHTAG = "#kegbot"

def get_api(oauth_token, oauth_token_secret):
  auth = tweepy.OAuthHandler(_CONSUMER_KEY, _CONSUMER_SECRET)
  auth.set_access_token(oauth_token, oauth_token_secret)
  return tweepy.API(auth)

def update_site_status(site, tweet):
  return update_status(site.twitter_profile.oauth_token,
      site.twitter_profile.oauth_token_secret, tweet)

def update_status(oauth_token, oauth_token_secret, tweet):
  api = get_api(oauth_token, oauth_token_secret)
  return api.update_status(tweet, headers=HEADERS)

def get_user_profile(user):
  try:
    return sr_twitter_models.TwitterProfile.objects.get(user=user)
  except sr_twitter_models.TwitterProfile.DoesNotExist:
    return None

def get_site_profile(site):
  try:
    return models.SiteTwitterProfile.objects.get(site=site)
  except models.SiteTwitterProfile.DoesNotExist:
    return None
